/*
 * Programmer:  Timothy E. Hoff
 * File Name:   verify.c
 * Written:     July 7, 1996
 * Modified:
 * Description: This module performs user verification and authentication
 *              for the requested command resource.
 *
 * Copyright (c) 1996, Timothy E. Hoff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#ifdef _AIX
#include <userpw.h>
#elif defined (_SVR4)
#include <pwd.h>
#include <shadow.h>
#include <crypt.h> 
#endif /* AIX */
#include "sfw.h"
#include "util.h"

#ifndef _LINT
static char rcsid[] = "@(#) $Id: verify.c,v 2.2 1996/07/22 20:42:34 t07de7 Exp t07de7 $";
#endif

/*
 * Verification Steps:
 *
 * 1.  check day and time restrictions
 * 2.  check for expired privileges
 * 3.  check for terminal restrictions
 * 4.  if req_passwd, perform password authentication
 *
 */

int verify_user(struct ace *ace, char **tty_list, int req_password)
{
    /*
     * Return 0 if the user has the necessary privileges to access
     * the resource otherwise return -1.
     */

    time_t today;
    struct tm *ptr;
    char *terminal;
    int valid_term;

    time(&today);
    ptr = localtime(&today);

    if (! (ace->days_allowed[ptr->tm_wday] == 1 &&
        (ace->start_time <= today && ace->end_time >= today))) {
        fprintf(stderr, "Sorry.  Access is not allowed at this time.\n");
        return -1;
    }

    if (today >= ace->expiration) {
        fprintf(stderr, "Sorry.  Your access has expired.\n");
        return -1;
    }

    if (*tty_list == NULL) {            /* no terminal restrictions */
        valid_term = 1;
    } else {
        if ((terminal = getctty()) == NULL){    /* get controlling terminal */
            terminal = "daemon";                /* daemon process (ie. cron) */
        }
        valid_term = 0;
        while (*tty_list != NULL) {
            if (strcmp(*tty_list, terminal) == 0) {
                valid_term = 1;
                break;
            }
	    tty_list++;
        }

        if (valid_term == 0) {
            fprintf(stderr, "Sorry.  Access is not allowed from this terminal.\n");
            return -1;
        }
    }

    if (req_password == 1) {
        return auth_user(ace->login);
    }

    return 0;
}


int auth_user(char *login)
{
    /*
     * Authenticate login using the UNIX passwd file.  If the user
     * enters the correct password return 0 otherwise return -1.
     */

    
    char *passwd, *unix_pw, *salt, *crypt_pw;
    char prompt[64];

#ifdef _AIX
    struct userpw *pw;
#elif defined (_SVR4)
    struct spwd *pw;
#endif

    sprintf(prompt, "Enter %s's password: ", login);
    passwd = getpass(prompt);

#ifdef _AIX
    if ((pw = getuserpw(login)) != NULL) {
        unix_pw = pw->upw_passwd;
    } else {
        return -1;                          /* User not defined */
    }
#elif defined (_SVR4)
    if ((pw = getspnam(login)) != NULL) {
        unix_pw = pw->sp_pwdp;
    } else {
        return -1;
    }
#endif

    salt = unix_pw;
    crypt_pw = crypt(passwd, salt);

    if (strcmp(unix_pw, crypt_pw) == 0) {
        /*
         * This could be enhanced by checking for expired accounts.  It may
         * be possible under certain circumstances for a user with an expired
         * account to access a command resource.
         */
        return 0;
    } else {
        return -1;
    }
}
    
